unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, hyiedefs, hyieutils, iexBitmaps,
  iesettings, iemio, iexLayers, iexRulers, iexToolbars,
  iexUserInteractions, imageenio, imageenproc, iexProcEffects, imageenview,
  ieview, iemview, iexFolderMView, Vcl.ExtCtrls, Vcl.ComCtrls,
  System.Actions, Vcl.ActnList, Vcl.StdCtrls, iexActions, iexActionsLayers,
  iexLayerMView, System.ImageList, Vcl.ImgList;

type
  TForm1 = class(TForm)
    ImageEnView1: TImageEnView;
    ImageEnFolderMView1: TImageEnFolderMView;
    Splitter1: TSplitter;
    StatusBar1: TStatusBar;
    PanelMain: TPanel;
    PanelMainBottom: TPanel;
    ButtonPromptToSaveSelectedLayerAs: TButton;
    ActionList1: TActionList;
    ImageEnViewLayersPromptToSave1: TImageEnViewLayersPromptToSave;
    IELayerMView: TImageEnLayerMView;
    PanelMainRight: TPanel;
    PanelLeft: TPanel;
    ButtonAddSelectedLayerAs: TButton;
    LabelInfo: TLabel;
    ImageList32: TImageList;
    ImageEnViewLayersSave1: TImageEnViewLayersSave;
    ComboBoxCreateLayers: TComboBox;
    LabelCreateLayer: TLabel;
    Splitter2: TSplitter;
    ButtonDeleteLayerFile: TButton;
    ButtonGroupSelectedLayers: TButton;
    ButtonMergeSelectedLayers: TButton;
    ButtonLoadImage: TButton;
    ButtonSaveImageAs: TButton;
    LabelLayers: TLabel;
    procedure ButtonAddSelectedLayerAsClick(Sender: TObject);
    procedure ButtonDeleteLayerFileClick(Sender: TObject);
    procedure ButtonGroupSelectedLayersClick(Sender: TObject);
    procedure ButtonLoadImageClick(Sender: TObject);
    procedure ButtonMergeSelectedLayersClick(Sender: TObject);
    procedure ButtonPromptToSaveSelectedLayerAsClick(Sender: TObject);
    procedure ButtonSaveImageAsClick(Sender: TObject);
    procedure ComboBoxCreateLayersChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ImageEnFolderMView1EndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure ImageEnFolderMView1KeyUp(Sender: TObject; var Key: Word; Shift:
        TShiftState);
    procedure ImageEnFolderMView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure ImageEnFolderMView1SelectionChanged(Sender: TObject);
    procedure ImageEnView1Click(Sender: TObject);
    procedure ImageEnView1DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ImageEnView1DragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ImageEnView1LayerNotifyEx(Sender: TObject; layer: Integer; event: TIELayerEvent);
    procedure ImageEnView1LayerSelectionChange(Sender: TObject);
  private
    { Private declarations }
    const
      FLayerBackgroundName = '_Background';
    var
      FExeFolder: string;

    procedure EnableSaveLayerButtons(AValue: Boolean);
    procedure TryDeleteMViewItem;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  CodeSiteLogging,
  System.IOUtils,
  iegdiplus;

{$R *.dfm}

procedure TForm1.ButtonAddSelectedLayerAsClick(Sender: TObject);
// save the Layer in the Exe folder
begin
  var LValue: string := '';
  repeat
    if not InputQuery('Add selected Layer to the Collection', 'Layer Name:', LValue) then EXIT;
    // Todo: also validate valid chars, existing file
  until Trim(LValue) <> '';

  var TempIEV := TImageEnView.Create(nil);
  try
    var idx := TempIEV.LayersAdd(ImageEnView1.CurrentLayer);
    TempIEV.Layers[idx].Name := LValue;
    TempIEV.Layers[0].Name := FLayerBackgroundName;
    TempIEV.LayersCropBackground();
    var LFile := IncludeTrailingPathDelimiter(FExeFolder) + LValue + '.ien';
    // Todo: Handle existing file
    TempIEV.IO.SaveToFileIEN(LFile);
    ImageEnFolderMView1.RefreshFileList;
  finally
    TempIEV.Free;
  end;
end;

procedure TForm1.ButtonDeleteLayerFileClick(Sender: TObject);
begin
  TryDeleteMViewItem;
end;

procedure TForm1.ButtonGroupSelectedLayersClick(Sender: TObject);
begin
  // Group all selected layers:
  ImageEnView1.LayersGroup();
end;

procedure TForm1.ButtonLoadImageClick(Sender: TObject);
begin
  var ThisFilename := ImageEnView1.IO.ExecuteOpenDialog();
  if ThisFilename <> '' then
  begin
    ImageEnView1.IO.LoadFromFileAuto(ThisFilename);
  end;
end;

procedure TForm1.ButtonMergeSelectedLayersClick(Sender: TObject);
begin
  // Merge all selected layers:
  ImageEnView1.LayersMerge();
end;

procedure TForm1.ButtonPromptToSaveSelectedLayerAsClick(Sender: TObject);
// save the Layer in a custom location
begin
  ImageEnView1.Layers[0].Name := FLayerBackgroundName;
  ImageEnView1.LayersCropBackground(True);
  ButtonPromptToSaveSelectedLayerAs.Action.Execute; // prompt to save ien file
end;

procedure TForm1.ButtonSaveImageAsClick(Sender: TObject);
begin
  var sFilename := ImageEnView1.IO.ExecuteSaveDialog({'Save Image', ioPNG});
  if sFilename <> '' then
    ImageEnView1.IO.SaveToFile(sFileName);
end;

procedure TForm1.ComboBoxCreateLayersChange(Sender: TObject);
begin
  case ComboBoxCreateLayers.ItemIndex of
    0: // layer from Image File
      begin
        ImageEnView1.LayersNewLayerDialog(ielkImage, True);
        ImageEnView1.CurrentLayer.AspectRatioLocked := True;
      end;
    1: // Simple Line
      begin
        ImageEnView1.MouseInteractLayers := [mlClickCreateLineLayers, mlEditLayerPoints];
      end;
    2: // Create an Angle
      begin
        ImageEnView1.LayersNewLayerDialog(ielkAngle, True);
      end;
    3: // Create a Shape
      begin
        ImageEnView1.LayersNewLayerDialog(ielkShape, True);
      end;
    4: // New Text Layer
      begin
        ImageEnView1.LayersAdd(ielkText);
        with TIETextLayer(ImageEnView1.CurrentLayer) do
        begin
          BorderColor := clBlack;
          BorderWidth := 1;
          FillColor := clInfoBk;
          AutoSize := True;
          Height := 40;
          Layout := ielCenter;
          Alignment := iejCenter;
          ActivateEditor;
        end;
        ImageEnView1.Update();
      end;
    5: // Transparent Glass
      begin
        ImageEnView1.LayersAdd(ielkText, 100, 100);
        var txtLayer := TIETextLayer(ImageEnView1.CurrentLayer);
        txtLayer.Font.Size := 28;
        txtLayer.Font.Style := [fsBold];
        txtLayer.Font.Color := clWhite;
        txtLayer.TextStyling.BorderWidth := 3;
        txtLayer.Alignment := iejCenter;
        txtLayer.TextStyling.BorderColor := clGray;
        txtLayer.TextStyling.BorderLineJoin := ieljRound;
        txtLayer.TextStyling.FillTransparency1 := 0; // 100% transparent
        txtLayer.Text := 'Buy' +#13#10+ 'Transparent' +#13#10+ 'Glass!' +#13#10+ '   ';
        txtLayer.SizeToText();
        ImageEnView1.Update();
      end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FExeFolder := ExtractFileDir(Application.ExeName);
  ImageEnFolderMView1.Folder := FExeFolder;
  ImageEnView1.Layers[0].Width  := 700;
  ImageEnView1.Layers[0].Height := 500;
end;

procedure TForm1.ImageEnFolderMView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
// beginning the drag operation in the MultiView
begin
  if ssLeft in Shift then // if the left mouse button is held down
  begin
    ImageEnFolderMView1.MouseInteract := []; // Clicking the mouse will not select images anymore
    // begin the drag operation from ImageEnMView1 (it is finished in ImageEnMView1EndDrag event):
    ImageEnFolderMView1.IEBeginDrag( False, -1 ); // ImageEn's implementation of BeginDrag
  end;
end;

procedure TForm1.ImageEnFolderMView1EndDrag(Sender, Target: TObject; X, Y: Integer);
// the drag operation from ImageEnMView1 has finished
begin
  ImageEnFolderMView1.IEEndDrag; // ImageEn's implementation of EndDrag. You must use this for Drag/Drop operations in TImageEnMView
  ImageEnFolderMView1.MouseInteract := [mmiSelect]; // Clicking the mouse will again select images
end;

procedure TForm1.ImageEnView1DragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
// while dragging the ImageEnFolderMView1 image over ImageEnView1
begin
  if (Source is TImageEnFolderMView) then // if the dragged image comes from ImageEnFolderMView1
    Accept := True; // drag cursor is shown
end;

procedure TForm1.ImageEnView1DragDrop(Sender, Source: TObject; X, Y: Integer);
// the ImageEnMView1 image has been dropped on ImageEnView1
var
  bmp: TIEBitmap;
  idx: Integer;
  LLayer: TIELayer;
begin
  If Source is TImageEnFolderMView then
  begin
    idx := TImageEnMView(Source).SelectedImage;
    if idx >= 0 then
    begin
      ImageEnView1.IO.LoadFromFileIEN(ImageEnFolderMView1.SelectedFilename, True);

      for var i := 1 to ImageEnView1.LayersCount - 1 do
      begin
        if SameText(ImageEnView1.Layers[i].Name, FLayerBackgroundName ) then
        begin
          ImageEnView1.LayersRemove(i); // remove old Background
          BREAK;
        end;
      end;

      ImageEnView1.CurrentLayer.PosX := ImageEnView1.XScr2Bmp(X);
      ImageEnView1.CurrentLayer.PosY := ImageEnView1.YScr2Bmp(Y);
      ImageEnView1.CurrentLayer.Selectable := True;
      ImageEnView1.CurrentLayer.Selected := True;
      EnableSaveLayerButtons(True);
      StatusBar1.SimpleText := ' A ' + ImageEnView1.CurrentLayer.KindStr + ' Layer has been dropped onto ImageView1.';
    end;
  end;

  ImageEnView1.Update();
end;

procedure TForm1.EnableSaveLayerButtons(AValue: Boolean);
begin
  ButtonPromptToSaveSelectedLayerAs.Enabled := AValue;
  ButtonAddSelectedLayerAs.Enabled          := AValue;
end;

procedure TForm1.ImageEnFolderMView1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_DELETE: TryDeleteMViewItem;
  end;
end;

procedure TForm1.ImageEnFolderMView1SelectionChanged(Sender: TObject);
begin
  ButtonDeleteLayerFile.Enabled := FileExists(ImageEnFolderMView1.SelectedFilename);
end;

procedure TForm1.TryDeleteMViewItem;
begin
  if ImageEnFolderMView1.SelectedImage < 0 then EXIT;

  var LSelectedFilePath := ImageEnFolderMView1.SelectedFilename;
  //CodeSite.Send('TForm1.TryDeleteMViewItem: LSelectedFilePath', LSelectedFilePath);

  if FileExists(LSelectedFilePath) then
  begin
    var SelectedFileName := System.IOUtils.TPath.GetFileNameWithoutExtension(LSelectedFilePath);
    //CodeSite.Send('TForm1.TryDeleteMViewItem: SelectedFileName', SelectedFileName);

    if Application.MessageBox(PChar('Delete the selected Layer:' + #13#10#13#10 + SelectedFileName), PChar('Application'),
      MB_OKCANCEL + MB_ICONQUESTION + MB_DEFBUTTON2 + MB_TOPMOST) = IDOK
    then
    begin
      //ImageEnFolderMView1.DeleteSelectedImages; // does not work on file level
      if not System.SysUtils.DeleteFile(LSelectedFilePath) then
        Application.MessageBox('The Layer could not be deleted.', PChar('Application'), MB_OK + MB_ICONSTOP + MB_DEFBUTTON2 + MB_TOPMOST);
    end;

    ImageEnFolderMView1.RefreshFileList;
  end;
end;

procedure TForm1.ImageEnView1Click(Sender: TObject);
begin
  // To cover also those cases where layers were deselected by clicking outside the Image Area:
  EnableSaveLayerButtons(ImageEnView1.LayersSelCount(False) > 0);
end;

procedure TForm1.ImageEnView1LayerNotifyEx(Sender: TObject; layer: Integer; event: TIELayerEvent);
begin
  if event = ielSelected then
    EnableSaveLayerButtons(layer > 0)
  else if event = ielRemoved then
    EnableSaveLayerButtons(False)
  else if event = ielCreated then
    ImageEnView1.MouseInteractLayers := [mlMoveLayers, mlResizeLayers, mlRotateLayers];
end;

procedure TForm1.ImageEnView1LayerSelectionChange(Sender: TObject);
begin
  var MoreThanOneLayersSelected := ImageEnView1.LayersSelCount(False) > 1;
  ButtonGroupSelectedLayers.Enabled := MoreThanOneLayersSelected;
  ButtonMergeSelectedLayers.Enabled := MoreThanOneLayersSelected;
end;

end.
